﻿using System;
using System.Globalization;

namespace Dynamicweb.Ecommerce.LiveIntegration.Extensions
{    
    /// <summary>
    /// Extensions methods for formatting of dates and numbers.
    /// </summary>
    internal static class IntegrationExtensions
    {        
        /// <summary>
        /// Rounds the value to the specified number of decimals and then formats it using <see cref="GetNumberFormatter" />.
        /// </summary>
        /// <param name="value">The value to process.</param>
        /// <param name="numberOfDecimals">The number of decimals.</param>
        /// <returns>System.String.</returns>
        internal static string ToIntegrationString(this double value, int numberOfDecimals = 2)
        {
            return Math.Round(value, numberOfDecimals).ToString(GetNumberFormatter());
        }

        /// <summary>
        /// Formats the date time to a sortable date/time format.
        /// </summary>
        /// <param name="value">The value to process.</param>        
        /// <returns>System.String.</returns>
        internal static string ToIntegrationString(this DateTime value)
        {
            return value.ToString("s");
        }

        internal static string ToIntegrationString(this object obj)
        {
            string value;
            if (obj is double)
            {
                value = ((double)obj).ToIntegrationString();
            }
            else if (obj is DateTime)
            {
                value = ((DateTime)obj).ToIntegrationString();
            }
            else
            {
                value = obj.ToString();
            }
            return value;
        }

        /// <summary>
        /// Gets the number formatter.
        /// </summary>
        /// <returns>NumberFormatInfo.</returns>
        private static NumberFormatInfo GetNumberFormatter()
        {
            CultureInfo culture = Helpers.GetCultureInfo();
            return culture != null ? culture.NumberFormat : NumberFormatInfo.InvariantInfo;
        }
    }
}